/********************************************************* {COPYRIGHT-TOP} ***
 * Licensed Materials - Property of IBM
 * 5725-Z22, 5725-Z63, 5725-U33, 5725-Z63
 *
 * (C) Copyright IBM Corporation 2016, 2017
 *
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ********************************************************** {COPYRIGHT-END} **/
// Node module: apiconnect

var exec = require('child_process').exec;
var f = require('util').format;
var fs = require('fs');
var path = require('path');
var rimraf = require('rimraf');
var test = require('tap').test;

process.env.APIC_CONFIG_PATH =
path.join(__dirname, 'sandbox', 'licenseTest' + '-' + Date.now());

var LICENSEACCEPTED_PATH = path.resolve(process.env.APIC_CONFIG_PATH, '.license-accepted');
var LICENSE_PATH = path.resolve(__dirname, path.join('..', 'node_modules', 'apiconnect-cli-plugins', 'LICENSE.txt'));
var apic = [ 'node', require.resolve('../bin/cli') ].join(' ');

test('cli behavior for License acceptance', function(t) {
  var options = { maxBuffer: 1000 * 1024 };
  var licenseTxt = f(
    'Please review the license for API Connect available in %s and select yes to accept',
    LICENSE_PATH
  );

  t.test('License file should be present', function(t) {
    fs.access(LICENSE_PATH, fs.F_OK, function(err) {
      t.error(err, 'file is accessible');
      t.end();
    });
  });

  t.test('Should display license: Do not accept', function(t) {
    execCmd(apic + ' -h', options, false, function(err, stdout, stderr) {
      t.error(err, 'Command executed successfully');
      t.match(stdout, new RegExp(licenseTxt), 'License was displayed');

      t.test('License not accepted: should not create .license-accepted file',
        function(t) {
          fs.access(LICENSEACCEPTED_PATH, fs.F_OK, function(err) {
            t.equal(err.code, 'ENOENT',
              'license-accepted file has not been created');
            t.end();
          });
        });
      t.end();
    });
  });

  t.test('Should display license: Accept', function(t) {
    execCmd(apic + ' -h', options, true, function(err, stdout, stderr) {
      t.error(err, 'Command executed successfully');
      t.match(stdout, new RegExp(licenseTxt), 'Displayed license and accepted');
      t.test('When license is accepted: should create .license-accepted file', function(t) {
        fs.access(LICENSEACCEPTED_PATH, fs.F_OK, function(err) {
          t.error(err, 'license file is created');
          t.end();
        });
      });

      t.test('Should not display license once accepted', function(t) {
        execCmd(apic + ' -h', options, true, function(err, stdout, stderr) {
          t.error(err, 'Command executed successfully');
          t.notMatch(stdout, new RegExp(licenseTxt));
          t.end();
        });
      });
      t.end();
    });
  });

  t.end();
  t.tearDown(function() { //clean up after test
    rimraf(process.env.APIC_CONFIG_PATH, function(err) {
      if (err) {
        throw err;
      }
    });
  });
});

function execCmd(cmd, options, acceptLicense, cb) {
  var child = exec(cmd, options, cb);
  child.stdout.once('data', function(data) {
    if (acceptLicense) {
      child.stdin.write('\n');
    }
    child.stdin.end(); // ending write stream terminates child waiting for user input
  });
  return child;
}

