/********************************************************* {COPYRIGHT-TOP} ***
 * Licensed Materials - Property of IBM
 * 5725-Z22, 5725-Z63, 5725-U33, 5725-Z63
 *
 * (C) Copyright IBM Corporation 2016, 2017
 *
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ********************************************************** {COPYRIGHT-END} **/
// Node module: apiconnect

'use strict';

var Promise = require('bluebird');
var Stats = require('fast-stats').Stats;
var _ = require('lodash');
var fmt = require('util').format;
var fs = require('fs');
var path = require('path');
var test = require('tap').test;

var pluginLoadDir = path.join(__dirname, '..', 'node_modules');

test('Check plugin load time', function(t) {
  return getPlugins()
    .then(loadPlugins)
    .then(function(results) {
      var data = new Stats().push(_.map(results, function(t) {
        return t.time;
      }));
      // Allow for some amount of variance due to slow CI machines
      var median = data.median();
      var stddev = data.stddev() * 5;
      var max = median + stddev;
      t.comment('All commands must load in less than %s seconds.', max);
      results.forEach(function(result) {
        var msg = '%s took %s to load.';
        t.ok(result.topics);
        t.ok(result.time <= max, fmt(msg, result.pkg, result.time));
      });
    });
});

function getPlugins() {
  return new Promise(function(resolve, reject) {
    fs.readdir(pluginLoadDir, function(err, res) {
      if (err) {
        return reject(err);
      }
      var plugins = [];
      res.forEach(function(f) {
        try {
          var pkg = require(path.join(f, 'package.json'));
          if (pkg && _.indexOf(pkg.keywords, 'apic-toolkit-plugin') > -1) {
            plugins.push({ loc: f, pkg: pkg });
          }
        } catch (err) {
        }
      });
      return resolve(plugins);
    });
  });
}

function loadPlugins(plugins) {
  return Promise.map(plugins, function(plugin) {
    var start = t();
    var main = path.join(pluginLoadDir, plugin.loc, plugin.pkg.main);
    var maybePlugin = require(main);
    var topics = maybePlugin.getTopics ? maybePlugin.getTopics() : [];
    var total = (t() - start) / 1000.0;
    return { time: total, pkg: plugin.loc, topics: topics };
  }, { concurrency: 1 }); // limit concurrency as we only want to load one at a time
}

function t() {
  return new Date().getTime();
}

